@@ -10,41 +10,43 @@
 
 from __future__ import absolute_import, division, unicode_literals
 
-from jx_base.expressions._utils import simplified
-from jx_base.expressions.boolean_op import BooleanOp
-from jx_base.expressions.expression import Expression, NULL
+from jx_base.expressions.expression import Expression
 from jx_base.expressions.false_op import FALSE
 from jx_base.expressions.true_op import TRUE
 from jx_base.language import is_op
-from mo_dots import is_many
-from mo_future import zip_longest
-from mo_imports import expect
-from mo_json import BOOLEAN
+from mo_imports import expect, export
+from mo_json.types import T_BOOLEAN
 
-NotOp, OrOp = expect("NotOp", "OrOp")
+NotOp, OrOp, ToBooleanOp = expect("NotOp", "OrOp", "ToBooleanOp")
 
 
 class AndOp(Expression):
-    data_type = BOOLEAN
-    zero = TRUE  # ADD THIS TO terms FOR NO EEFECT
+    _data_type = T_BOOLEAN
+    default = TRUE  # ADD THIS TO terms FOR NO EEFECT
 
-    def __init__(self, terms):
-        Expression.__init__(self, terms)
-        if terms == None:
-            self.terms = []
-        elif is_many(terms):
-            self.terms = terms
-        else:
-            self.terms = [terms]
+    def __init__(self, *terms):
+        Expression.__init__(self, *terms)
+        self.terms = terms
 
     def __data__(self):
         return {"and": [t.__data__() for t in self.terms]}
 
+    def __call__(self, row, rownum=None, rows=None):
+        for a in self.terms:
+            if not a(row, rownum, rows):
+                return False
+        return True
+
     def __eq__(self, other):
         if is_op(other, AndOp):
-            return all(a == b for a, b in zip_longest(self.terms, other.terms))
+            return all(o in self.terms for o in other.terms) and all(
+                s in other.terms for s in self.terms
+            )
         return False
 
+    def __rcontains__(self, superset):
+        return any(t in superset for t in self.terms)
+
     def vars(self):
         output = set()
         for t in self.terms:
@@ -52,24 +54,20 @@ class AndOp(Expression):
         return output
 
     def map(self, map_):
-        return self.lang[AndOp([t.map(map_) for t in self.terms])]
+        return AndOp(*(t.map(map_) for t in self.terms))
 
-    def missing(self):
+    def missing(self, lang):
         return FALSE
 
-    def invert(self):
-        return self.lang[OrOp([t.invert() for t in self.terms])].partial_eval()
+    def invert(self, lang):
+        return OrOp(*(t.invert(lang) for t in self.terms)).partial_eval(lang)
 
-    @simplified
-    def partial_eval(self):
+    def partial_eval(self, lang):
         # MERGE IDENTICAL NESTED QUERIES
         # NEST DEEP NESTED QUERIES
         or_terms = [[]]  # LIST OF TUPLES FOR or-ing and and-ing
         for i, t in enumerate(self.terms):
-            simple = self.lang[BooleanOp(t)].partial_eval()
-            if simple.type != BOOLEAN:
-                simple = simple.exists()
-
+            simple = ToBooleanOp(t).partial_eval(lang)
             if simple is TRUE:
                 continue
             elif simple is FALSE:
@@ -79,7 +77,7 @@ class AndOp(Expression):
                     for tt in simple.terms:
                         if tt in and_terms:
                             continue
-                        if self.lang[NotOp(tt)].partial_eval() in and_terms:
+                        if (NotOp(tt)).partial_eval(lang) in and_terms:
                             or_terms.remove(and_terms)
                             break
                         and_terms.append(tt)
@@ -89,11 +87,11 @@ class AndOp(Expression):
                     and_terms + ([o] if o not in and_terms else [])
                     for o in simple.terms
                     for and_terms in or_terms
-                    if self.lang[NotOp(o)].partial_eval() not in and_terms
+                    if NotOp(o).partial_eval(lang) not in and_terms
                 ]
                 continue
             for and_terms in list(or_terms):
-                inv = self.lang[NotOp(simple)].partial_eval()
+                inv = NotOp(simple).partial_eval(lang)
                 if inv in and_terms:
                     or_terms.remove(and_terms)
                 elif simple not in and_terms:
@@ -107,13 +105,13 @@ class AndOp(Expression):
             elif len(and_terms) == 1:
                 return and_terms[0]
             else:
-                return self.lang[AndOp(and_terms)]
+                return AndOp(*and_terms)
 
-        return self.lang[
-            OrOp(
-                [
-                    AndOp(and_terms) if len(and_terms) > 1 else and_terms[0]
-                    for and_terms in or_terms
-                ]
-            )
-        ].partial_eval()
+        return OrOp(*(
+            AndOp(*and_terms) if len(and_terms) > 1 else and_terms[0]
+            for and_terms in or_terms
+        )).partial_eval(lang)
+
+
+export("jx_base.expressions.expression", AndOp)
+export("jx_base.expressions.base_multi_op", AndOp)
