@@ -10,24 +10,24 @@
 
 from __future__ import absolute_import, division, unicode_literals
 
-from jx_base.expressions._utils import builtin_ops, simplified
+from jx_base.expressions._utils import builtin_ops
 from jx_base.expressions.expression import Expression
 from jx_base.expressions.false_op import FALSE
-from jx_base.expressions.literal import Literal
-from jx_base.expressions.literal import is_literal
+from jx_base.expressions.literal import is_literal, Literal
 from jx_base.expressions.variable import Variable
 from jx_base.language import is_op
-from mo_json import BOOLEAN
+from mo_json.types import T_BOOLEAN
 
 
 class BaseInequalityOp(Expression):
     has_simple_form = True
-    data_type = BOOLEAN
+    _data_type = T_BOOLEAN
     op = None
 
-    def __init__(self, terms):
-        Expression.__init__(self, terms)
-        self.lhs, self.rhs = terms
+    def __init__(self, lhs, rhs):
+        Expression.__init__(self, lhs, rhs)
+        self.lhs = lhs
+        self.rhs = rhs
 
     @property
     def name(self):
@@ -50,15 +50,14 @@ class BaseInequalityOp(Expression):
     def map(self, map_):
         return self.__class__([self.lhs.map(map_), self.rhs.map(map_)])
 
-    def missing(self):
+    def missing(self, lang):
         return FALSE
 
-    @simplified
-    def partial_eval(self):
-        lhs = self.lhs.partial_eval()
-        rhs = self.rhs.partial_eval()
+    def partial_eval(self, lang):
+        lhs = self.lhs.partial_eval(lang)
+        rhs = self.rhs.partial_eval(lang)
 
         if is_literal(lhs) and is_literal(rhs):
             return Literal(builtin_ops[self.op](lhs, rhs))
 
-        return self.__class__([lhs, rhs])
+        return self.__class__(lhs, rhs)
