@@ -11,14 +11,13 @@
 from __future__ import absolute_import, division, unicode_literals
 
 from jx_base.expressions.expression import Expression
-from jx_base.expressions.false_op import FALSE
-from mo_json import BOOLEAN
+from mo_json import T_INTEGER
 
 
 class IsIntegerOp(Expression):
-    data_type = BOOLEAN
+    _data_type = T_INTEGER
 
-    def __init__(self, term):
+    def __init__(self, *term):
         Expression.__init__(self, [term])
         self.term = term
 
@@ -29,7 +28,15 @@ class IsIntegerOp(Expression):
         return self.term.vars()
 
     def map(self, map_):
-        return self.lang[IsIntegerOp(self.term.map(map_))]
+        return IsIntegerOp(self.term.map(map_))
 
-    def missing(self):
-        return FALSE
+    def missing(self, lang):
+        return self.expr.missing()
+
+    def partial_eval(self, lang):
+        term = self.term.partial_eval(lang)
+
+        if term.type in T_INTEGER:
+            return term
+        else:
+            return NULL
