@@ -10,10 +10,9 @@
 from __future__ import absolute_import, division, unicode_literals
 
 from jx_base.expressions import FirstOp as FirstOp_
-from jx_python.expressions._utils import Python
 
 
 class FirstOp(FirstOp_):
-    def to_python(self, not_null=False, boolean=False, many=False):
-        value = Python[self.term].to_python()
+    def to_python(self):
+        value = self.term.to_python()
         return "listwrap(" + value + ")[0]"
